@@ -205,12 +205,13 @@ static int log_printf(const char *fmt, ...)
 	char buffer[4096];
 	char tmbuffer[10];
 	va_list ap;
-	va_start(ap, fmt);
 	log_item_t *log_item;
 	size_t len;
 
+	va_start(ap, fmt);
 	time_now(tmbuffer, sizeof(tmbuffer));
 	vsnprintf(buffer, sizeof(buffer), fmt, ap);
+	va_end(ap);
 
 	if ((log_item = calloc(1, sizeof(log_t))) == NULL) {
 		fprintf(stderr, "Out of memory allocating log item\n");
@@ -220,15 +221,12 @@ static int log_printf(const char *fmt, ...)
 	log_item->text = calloc(1, len);
 	snprintf(log_item->text, len, "%s%s", tmbuffer, buffer);
 
-	if (infolog.head == NULL) {
+	if (infolog.head == NULL)
 		infolog.head = log_item;
-		infolog.tail = log_item;
-	} else {
+	else
 		infolog.tail->next = log_item;
-		infolog.tail = log_item;
-	}
 
-	va_end(ap);
+	infolog.tail = log_item;
 
 	return 0;
 }
@@ -286,6 +284,8 @@ static int netlink_connect(void)
 	struct sockaddr_nl addr;
 
     	if ((sock = socket(PF_NETLINK, SOCK_DGRAM, NETLINK_CONNECTOR)) < 0) {
+		if (errno == EPROTONOSUPPORT)
+			return -EPROTONOSUPPORT;
 		fprintf(stderr, "Socket failed: %s\n", strerror(errno));
 		return -1;
 	}
@@ -297,6 +297,7 @@ static int netlink_connect(void)
 
 	if (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
 		fprintf(stderr, "Bind failed: %s\n", strerror(errno));
+		close(sock);
 		return -1;
 	}
 
@@ -642,6 +643,7 @@ static int power_rate_get_sys_fs(double *rate, bool *discharging, bool *inaccura
 			snprintf(path, sizeof(path), "%s/%s/uevent", SYS_CLASS_POWER_SUPPLY, dirent->d_name);
 			if ((fp = fopen(path, "r")) == NULL) {
 				fprintf(stderr, "Battery %s present but under supported - no state present.", dirent->d_name);
+				closedir(dir);
 				return -1;
 			} else {
 				char buffer[4096];
@@ -929,6 +931,7 @@ static int power_rate_get(double *rate, bool *discharging, bool *inaccurate)
 	    S_ISDIR(buf.st_mode))
 		return power_rate_get_proc_acpi(rate, discharging, inaccurate);
 
+	fprintf(stderr, "Machine does not seem to have a battery, cannot measure power.\n");
 	return -1;
 }
 
@@ -1022,6 +1025,18 @@ static int proc_info_add(const pid_t pid)
 	proc_info_t *info;
 	char path[PATH_MAX];
 	char cmdline[1024];
+	bool free_slot = false;
+
+	i = proc_info_hash(pid);
+	for (j = 0; j < MAX_PIDS; j++, i = (i + 1) % MAX_PIDS) {
+		if (proc_info[i] == NULL) {
+			free_slot = true;
+			break;
+		}
+	}
+
+	if (!free_slot)
+		return -1;
 
 	memset(cmdline, 0, sizeof(cmdline));	/* keep valgrind happy */
 
@@ -1041,13 +1056,8 @@ static int proc_info_add(const pid_t pid)
 	}
 	strcpy(info->cmdline, cmdline);
 
-	i = proc_info_hash(pid);
-	for (j = 0; j < MAX_PIDS; j++, i = (i + 1) % MAX_PIDS) {
-		if (proc_info[i] == NULL) {
-			proc_info[i] = info;
-			return 0;
-		}
-	}
+	proc_info[i] = info;
+
 	return -1;
 }
 
@@ -1114,6 +1124,8 @@ static int monitor(const int sock)
 
 		gettimeofday(&t2, NULL);
 		usec = ((t1.tv_sec + sample_delay - t2.tv_sec) * 1000000) + (t1.tv_usec - t2.tv_usec);
+		if (usec < 0)
+			goto sample_now;
 		tv.tv_sec = usec / 1000000;
 		tv.tv_usec = usec % 1000000;
 
@@ -1139,6 +1151,7 @@ static int monitor(const int sock)
 			char tmbuffer[10];
 			bool discharging;
 
+sample_now:
 			if (redone) {
 				char buffer[80];
 				int indent;
@@ -1263,7 +1276,7 @@ static int monitor(const int sock)
 					if (proc_ev->event_data.exit.process_pid ==
 					    proc_ev->event_data.exit.process_tgid)
 						proc_info_free(proc_ev->event_data.exit.process_pid);
-						redo = true;
+					redo = true;
                				break;
 				default:
                				break;
@@ -1429,13 +1442,19 @@ int main(int argc, char * const argv[])
 
 	log_init();
 	if (opts & OPTS_USE_NETLINK) {
-		proc_info_load();
-
-    		if ((sock = netlink_connect()) < 0)
+    		sock = netlink_connect();
+		if (sock == -EPROTONOSUPPORT) {
+			if (opts & OPTS_SHOW_PROC_ACTIVITY)
+				printf("Cannot show process activity with this kernel.\n");
+			opts &= ~OPTS_USE_NETLINK;
+		} else if (sock < 0) {
 			goto abort;
+		} else {
+			proc_info_load();
 
-		if (netlink_listen(sock) < 0)
-			goto abort_sock;
+			if (netlink_listen(sock) < 0)
+				goto abort_sock;
+		}
 	}
 
 	if (power_rate_get(&dummy_rate, &discharging, &dummy_inaccurate) < 0)
@@ -1451,7 +1470,8 @@ abort:
 	if (opts & OPTS_USE_NETLINK) {
 		log_dump();
 		log_free();
-		close(sock);
+		if (sock != -1)
+			close(sock);
 	}
 
 	exit(ret);
